﻿##### Scriptinfo ##################################################################################
# Scriptreihe:      Show-Diagramme
# Datum:            2016-02-09
# Version:          V1.00
# Programmierer:    Stephan Walther
###################################################################################################

# Funktionen
    function erstelle-Diagramm {
        param (
            [cmdletbinding()]        
            [parameter(Mandatory=$false)] [switch] $KeineLegende = $false,
            [parameter(Mandatory=$false)] [int]    $Breite = 1200, 
            [parameter(Mandatory=$false)] [int]    $Hoehe = 500,
            [parameter(Mandatory=$false)] [int]    $Y_Min = 0,
            [parameter(Mandatory=$false)] [int]    $Y_Max = 0,
            [parameter(Mandatory=$false)] [string] $Titel = '',
            [parameter(Mandatory=$false)] [Object] $Daten,
            [parameter(Mandatory=$false)] [string] $X_Property,
            [parameter(Mandatory=$false)] [validateset('Saeule','Linie','Kurve','Balken','Saeule_Stapel','Balken_Stapel','Saeule_Stapel_100','Balken_Stapel_100','Flaeche','Doughnut','Kreis','Radar')] [string] $DiagrammTyp = "Linie",
            [parameter(Mandatory=$false)] [string] $SpeichernInDatei = ""
        )

        # lade Bibliotheken    
        [void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 
        [void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms.DataVisualization")

        # erstelle Diagramm-Objekt    
        $Diagramm = New-object System.Windows.Forms.DataVisualization.Charting.Chart 
        $Diagramm.Width = $Breite
        $Diagramm.Height = $Hoehe
        $Diagramm.Left = 0 
        $Diagramm.Top = 0
        if ($DiagrammTitel -ne '') {
            $DiagrammTitel = New-Object System.Windows.Forms.DataVisualization.Charting.Title
            $DiagrammTitel.Text = $Titel
            $Diagramm.Titles.Add($DiagrammTitel)
        }

        # erstelle Zeichenoberflaeche
        $DiagrammArea = New-Object System.Windows.Forms.DataVisualization.Charting.ChartArea 
        $Diagramm.ChartAreas.Add($DiagrammArea)    
        $DiagrammArea.AxisX.Interval = 1
    
        $DiagrammArea.AxisY.Minimum = $y_min
        $DiagrammArea.AxisY.Maximum = $y_max 
        
        # fuege Legende an
        if ($KeineLegende -eq $false) {
            $Legende = New-Object System.Windows.Forms.DataVisualization.Charting.Legend
            $Diagramm.Legends.Add($Legende)
        }   
    
        if ($Daten) {
            $Diagramm | 
                anfuegen-DiagrammDatenreihe -Daten $Daten -X_Property $X_Property -DiagrammTyp $DiagrammTyp | 
                    zeige-Diagramm -SpeichernInDatei $SpeichernInDatei
        } else {
            return $Diagramm 
        }    
    }

    function anfuegen-DiagrammDatenreihe {
        param (
            [cmdletbinding()]
            [parameter(Mandatory=$true,ValueFromPipeline=$true)] $Diagramm,
            [parameter(Mandatory=$true)]  [Object] $Daten,
            [parameter(Mandatory=$true)]  [string] $X_Property,
            [parameter(Mandatory=$false)] [string] $ReihenName = '',
            [parameter(Mandatory=$false)] [validateset('Saeule','Linie','Kurve','Balken','Saeule_Stapel','Balken_Stapel','Saeule_Stapel_100','Balken_Stapel_100','Flaeche','Doughnut','Kreis','Radar')] [string] $DiagrammTyp = "Linie"
        )

        # lade Bibliotheken    
        [void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 
        [void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms.DataVisualization")
        
        # definiere Diagrammtyp
        switch ($DiagrammTyp) {
            'Saeule'           {$DiagrammType =  [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Column'}
            'Linie'            {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Line'}
            'Kurve'            {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Spline'}
            'Balken'           {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Bar'}
            'Saeule_Stapel'    {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'StackedColumn'}
            'Balken_Stapel'    {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'StackedBar'}
            'Saeule_Stapel_100'{$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'StackedColumn100'}
            'Balken_Stapel_100'{$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'StackedBar100'}
            'Flaeche'          {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'area'}
            'Doughnut'         {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Doughnut'}
            'Kreis'            {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'Pie'}
            'Radar'            {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'radar'}
            default            {$DiagrammType  = [System.Windows.Forms.DataVisualization.Charting.SeriesChartType]::'radar'}
        }
    
        # sammle Daten aus Eingabemenge -> je Spalte wird eine Datenreihe erstellt   
        $Ymin = 0
        $ymax = 1

        if ($ReihenName -eq '') {
            $ReihenName = Get-Random
            $LegendeReihenName = ""
        } else {
            $LegendeReihenName = " ($ReihenName)"
        }
        $Daten | Get-Member -MemberType Properties | Where-Object {$_.name -ne $X_Property} | ForEach-Object {
            $Spaltenname = $_.name

            $DataCol = [ordered]@{}
            $Daten | ForEach-Object {
                $DataCol[$_.$X_Property] = $_.$Spaltenname 
                if ($_.$Spaltenname -lt $Ymin) {$Ymin = $_.$Spaltenname}
                if ($_.$Spaltenname -gt $Ymax) {$Ymax = $_.$Spaltenname}
            }
        
            [void]$Diagramm.Series.Add("$ReihenName$Spaltenname") 
            $Werte = $Diagramm.Series["$ReihenName$Spaltenname"]
            $Werte.Points.DataBindXY($DataCol.Keys,$DataCol.Values)
            $Werte.LegendText = "$Spaltenname$LegendeReihenName"
            $Werte.ChartType = $DiagrammType
            $werte.IsValueShownAsLabel = $false                
        }
    
        $DiagrammArea = $Diagramm.ChartAreas[0]  
    
        if ($DiagrammArea.AxisY.Minimum -ne 0) {
            if ($DiagrammArea.AxisY.Minimum -lt $Ymin) {$Ymin = $DiagrammArea.AxisY.Minimum}    
        }
        if ($DiagrammArea.AxisY.Maximum -ne 0) {
            if ($DiagrammArea.AxisY.Maximum -gt $Ymax) {'OK';$Ymax = $DiagrammArea.AxisY.Maximum}    
        }
      
        $DiagrammArea.AxisY.Minimum = [int] $ymin 
        $DiagrammArea.AxisY.Maximum = [int] $ymax + 2
        $DiagrammArea.AxisY.Interval = [int] (($ymax - 2) / 4) 
    
        return $Diagramm
    }

    function zeige-Diagramm {
        param (
            [cmdletbinding()]
            [parameter(Mandatory=$true,ValueFromPipeline=$true)] $Diagramm,
            [parameter(Mandatory=$false)] [string] $SpeichernInDatei = ""
        )

        # erstelle Formular fuer die Ausgabe
        $Diagramm.Anchor = [System.Windows.Forms.AnchorStyles]::Bottom `
            -bor [System.Windows.Forms.AnchorStyles]::Right `
            -bor [System.Windows.Forms.AnchorStyles]::Top `
            -bor [System.Windows.Forms.AnchorStyles]::Left 
        $Form = New-Object Windows.Forms.Form 
        $Form.Text = "PowerShell Chart" 
        $Form.Width = $Diagramm.Width
        $Form.Height = $Diagramm.Height + 50
        $Form.controls.add($Diagramm) 
        $Form.Add_Shown({$Form.Activate()}) 
    
        if ($SpeichernInDatei -ne "") {
            $Pfad = $SpeichernInDatei.Substring(0,$SpeichernInDatei.Length - 3) + 'png'        
            $Diagramm.SaveImage($Pfad,'png')
            $Form.Close()
        } else {
            $Form.ShowDialog() | Out-Null
        }
    }

# Beispieldaten
    $Tabelle = 1..10 | ForEach-Object {
        $out = New-Object -TypeName psobject
        $out | Add-Member -MemberType NoteProperty -Name Datum -Value (get-date (get-date).AddDays($_) -Format d)
        $out | Add-Member -MemberType NoteProperty -Name Col1 -Value (Get-Random -Minimum 1 -Maximum 20)
        $out | Add-Member -MemberType NoteProperty -Name Col2 -Value (Get-Random -Minimum 1 -Maximum 10)
        $out | Add-Member -MemberType NoteProperty -Name Col3 -Value (Get-Random -Minimum 1 -Maximum 30)
        $out | Add-Member -MemberType NoteProperty -Name Col4 -Value (Get-Random -Minimum 1 -Maximum 40)
        $out
    }

# Anwendung der Funktionen
    # einfaches Diagramm
        erstelle-Diagramm -Daten $Tabelle -X_Property 'Datum' -DiagrammTyp Saeule -Titel 'Test'

    # Diagramm mit unterschiedlichen Typen je Datenreihe
        erstelle-Diagramm -Titel 'Test' | 
            anfuegen-DiagrammDatenreihe -Daten ($Tabelle | Select-Object -Property 'Datum','col1') -X_Property 'Datum' -DiagrammTyp Kurve | 
            anfuegen-DiagrammDatenreihe -Daten ($Tabelle | Select-Object -Property 'Datum','col2') -X_Property 'Datum' -DiagrammTyp Saeule | 
                zeige-Diagramm

    # Datenausgabe als Datei
        erstelle-Diagramm -Daten $Tabelle -X_Property 'Datum' -DiagrammTyp Saeule -Titel 'Test' -SpeichernInDatei 'e:\diag.jpg'

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUc1DXjzlSf980u8U5J3nFnUmn
# tjegggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBTpcz0qE3UlKDO1AZ5hqvlG87M50TANBgkqhkiG9w0BAQEF
# AASCAQAeYLYpy/vRzoFhiOAtef7AVt1aagSqjFWvNBlA88GNLSxUi8DKLuoLSlx6
# w1Zgz9HKmVM9HJdgSSeeCTF9Ne1ZlGKoCp/BUluI5BpaQalL5c77dCVWcazQfN4/
# XC1O2Tkx/g364sFPDnhxBgpiwtSrarWbTZwpnVq2jDYaFR7yq7ZvKwbqTU17Srcj
# K1mQGHfBMJpU1YtLAD/q0fvzwY2+eUAKRr1aOncVoUAPejAOunbMcm27mK5otskO
# VQcB70yaw1AbqRa9o+VTSkwzGZ02JqU7yrODtcDJVyU5KqTP6/qaiFdN1AI0CEjj
# ucVSadwNNJg73nBx6FFTMFApGLCp
# SIG # End signature block
